/*
 * Object: Ivanti Security Controls Create Job Data Refresh v2022.4.1    Script Date: 11/15/2022 2:21:57 PM 
 * Use this script to create the job to refresh the Security Controls database.
 * Ensure the @DatabaseName and @JobOwner variables below are set to the appropriate values
 *
 */
USE [msdb]
GO

BEGIN TRANSACTION
DECLARE @DatabaseName NVARCHAR(100) = 'YOUR SECURITY CONTROLS DATABASE NAME'
DECLARE @JobOwner NVARCHAR(100) = 'sa'
DECLARE @JobName NVARCHAR(100) = @DatabaseName + ' data refresh'

DECLARE @ReturnCode INT
SELECT @ReturnCode = 0



IF NOT EXISTS (SELECT name FROM msdb.dbo.syscategories WHERE name=N'[Uncategorized (Local)]' AND category_class=1)
BEGIN
EXEC @ReturnCode = msdb.dbo.sp_add_category @class=N'JOB', @type=N'LOCAL', @name=N'[Uncategorized (Local)]'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

END


DECLARE @jobId BINARY(16)
EXEC @ReturnCode =  msdb.dbo.sp_add_job @job_name=@JobName, 
		@enabled=1, 
		@notify_level_eventlog=0, 
		@notify_level_email=0, 
		@notify_level_netsend=0,
		@notify_level_page=0, 
		@delete_level=0, 
		@description=N'Refreshes the CurrentPatchStatus and CurrentPatchCOUNT tables with the latest data', 
		@category_name=N'[Uncategorized (Local)]', 
		@owner_login_name=@JobOwner, @job_id = @jobId OUTPUT
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback





EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'Check Procedures Exist', 
		@step_id=1, 
		@cmdexec_success_code=0, 
		@on_success_action=3, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'

DECLARE @RETRYCOUNT INT = 1
DECLARE @RETRYMAX INT = 3
DECLARE @LASTBATCH INT
DECLARE @LASTERROR NVARCHAR(4000)


DECLARE @SQLCOUNT NVARCHAR(Max) = N''(SELECT @RETOUT = COUNT(*)  FROM information_schema.routines
WHERE ROUTINE_TYPE = ''''PROCEDURE''''
  AND ROUTINE_NAME LIKE ''''xtr%'''')''
  DECLARE @PARMDEFINITION NVARCHAR(500)
  DECLARE @TEST INT
  DECLARE @RETVAL INT
  SET @PARMDEFINITION = N''@RETOUT INT OUTPUT''


EXEC sp_executesql @SQLCOUNT, @PARMDEFINITION, @RETOUT = @RETVAL OUTPUT;

IF (@RETVAL < 14)



BEGIN

DECLARE @SQLProc NVARCHAR(Max)

IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_CurrentPatchCount]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_CurrentPatchCount] ;


IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_UpdateData]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_UpdateData] ;



IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_DeleteCPatches]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_DeleteCPatches] ;


IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_DeleteObsPatches]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_DeleteObsPatches] ;


IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_DeleteODPatches]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_DeleteODPatches] ;

IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_DistinctAssessed]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_DistinctAssessed] ;


IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_DistinctPatched]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_DistinctPatched] ;


IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_DistinctDeployed]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_DistinctDeployed] ;


IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_LogErrorInfo]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_LogErrorInfo] ;


IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_CurrentPatchStatus]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_CurrentPatchStatus] ;

	IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_CurrentDeployStatus]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_CurrentDeployStatus] ;

IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_LinuxDeleteCPatches]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_LinuxDeleteCPatches] ;


IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_LinuxUpdateData]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_LinuxUpdateData] ;


IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_LinuxCurrentPatchStatus]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_LinuxCurrentPatchStatus] ;


IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_LinuxCurrentPatchCount]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_LinuxCurrentPatchCount] ;


IF EXISTS ( SELECT  *
            FROM    sys.objects
            WHERE   object_ID = OBJECT_ID(N''[dbo].[xtr_LinuxDistinctDeployed]'')
                    AND type IN (N''P'') ) 
    DROP PROCEDURE [dbo].[xtr_LinuxDistinctDeployed] ;


SET @SQLProc = ''CREATE PROCEDURE [dbo].[xtr_LogErrorInfo] (
	@BATCHID INT
	, @ENTITYNAME NVARCHAR(100)
)
AS



INSERT INTO [dbo].[xtrEntityProcessErrorLog]
           ([BATCHID]
           ,[LOGDATE]
           ,[ENTITYNAME]
           ,[ERRORNUMBER]
           ,[ERRORSEVERITY]
           ,[ERRORSTATE]
           ,[ERRORPROCEDURE]
           ,[ERRORLINE]
           ,[ERRORMESSAGE])
SELECT
	@BATCHID
	, GETDATE()
	, @ENTITYNAME
	, ERROR_NUMBER() AS ERRORNUMBER
    , ERROR_SEVERITY() AS ERRORSEVERITY
    , ERROR_STATE() AS ERRORSTATE
    , ERROR_PROCEDURE() AS ERRORPROCEDURE
    , ERROR_LINE() AS ERRORLINE
    , ERROR_MESSAGE() AS ERRORMESSAGE;
 

 INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 
	VALUES (@BATCHID, GETUTCDATE(),  ERROR_PROCEDURE(), ''''Error(s) encountered. See more details in table dbo.EntityProcessErrorLog'''')''

EXEC sp_executesql @SQLProc







	
SET @SQLProc = ''CREATE PROCEDURE [dbo].[xtr_CurrentPatchCount](
@BATCHID INT)

AS

DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_CurrentPatchCount''''

BEGIN TRY
TRUNCATE TABLE xtrCurrentPatchCount

INSERT INTO xtrCurrentPatchCount

SELECT AMS.MACHINEID AS MACHINE, VS.VALUE AS SEVERITY, PG.PGRPNAME AS [PATCHGROUP],
SUM(CASE WHEN INSTALLSTATEID = 3 THEN 1
WHEN INSTALLSTATEID = 4 AND PD.DeployStateId IN (5,8,13) THEN 1
ELSE 0 END) AS INSTALLED,
SUM(CASE WHEN INSTALLSTATEID = 4 AND PD.DeployStateId IN (1,2,3,4,7,39,42,95,50,192,193,194) THEN  1 
WHEN INSTALLSTATEID = 4 AND PD.DeployStateId IS NULL THEN  1 ELSE 0 END) AS NOTINSTALLED,
SUM(CASE WHEN INSTALLSTATEID = 6  AND PD.DeployStateId IS NULL THEN 1 ELSE 0 END) AS MISSINGSERVICEPACK,
SUM(CASE WHEN INSTALLSTATEID = 2 THEN 1 ELSE 0 END) AS INFORMATIONAL

FROM xtrCurrentPatchStatus PP
LEFT OUTER JOIN 
Reporting2.AssessedMachineState AMS ON AMS.ID = PP.ASSESSEDMACHINESTATEID
LEFT OUTER JOIN
Reporting2.Patch P ON PP.PATCHID = P.ID
LEFT OUTER JOIN
Dbo.PatchGroupPatch PGP ON PGP.PATCHID = P.ID
LEFT OUTER JOIN
Dbo.PatchGroup PG ON PG.PGRPKEY = PGP.PATCHGROUPID
LEFT OUTER JOIN
Reporting2.VendorSeverity VS ON VS.ID = P.VENDORSEVERITYID
LEFT OUTER JOIN [Reporting2].[PatchDeployment] PD
ON PD.DeployId = PP.DEPLOYID AND PD.DetectedPatchStateId = PP.DETECTEDPATCHSTATEID
GROUP BY AMS.MACHINEID,  VS.VALUE, PG.pgrpName


RETURN CAST(@@rowcount AS INT)

END TRY

BEGIN CATCH
	EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
	
END CATCH''



EXEC sp_executesql @SQLProc




SET @SQLProc = ''CREATE PROCEDURE [dbo].[xtr_CurrentDeployStatus]
(
@BATCHID INT,
@RECORDSSTAGED INT OUTPUT)

AS 
DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_CurrentDeployStatus''''
BEGIN TRY

IF OBJECT_ID(''''tempdb..#MergeSummary'''') IS NULL
	CREATE TABLE #MergeSummary (CHANGE VARCHAR(20))

IF OBJECT_ID(''''tempdb..#CurrentDeploy'''') IS NULL
CREATE TABLE #CurrentDeploy (
MACHINEID INT,
PATCHID INT,
PRODUCTID INT,
DEPLOYID INT NULL,
DETECTEDPATCHSTATEID INT NULL
)


INSERT INTO #CurrentDeploy
SELECT MS.MACHINEID AS MACHINEID, DPS.PATCHID AS PATCHID, PT.ID AS PRODUCTID, PD.DEPLOYID AS DEPLOYID, PD.DETECTEDPATCHSTATEID AS DETECTEDPATCHSTATEID
FROM Reporting2.AssessedMachineState MS
LEFT OUTER JOIN
Reporting2.PatchScan PS ON PS.ID = MS.PATCHSCANID
LEFT OUTER JOIN
Reporting2.Machine MA ON MS.MACHINEID = MA.ID
LEFT OUTER JOIN
Reporting2.DetectedPatchState DPS ON MS.ID = DPS.ASSESSEDMACHINESTATEID
LEFT OUTER JOIN
DBO.ScanMachines SM ON DPS.ASSESSEDMACHINESTATEID = SM.SMACHID
LEFT OUTER JOIN
Reporting2.Patch P ON DPS.PATCHID = P.ID
LEFT OUTER JOIN
Reporting2.VendorSeverity VS ON VS.ID = P.VENDORSEVERITYID
LEFT OUTER JOIN
Reporting2.Product PT ON DPS.PRODUCTID = PT.ID
LEFT OUTER JOIN
Reporting2.PatchDeployment PD
ON DPS.ID = PD.DETECTEDPATCHSTATEID
INNER JOIN(
SELECT AMS.MACHINEID,  P.ID AS PATCHID, PP.PRODUCTID AS PRODUCTID,  MAX(PD.DetectedPatchStateId) AS MAXAMS, 
MAX(PD.DEPLOYID) AS DEPLOY
FROM Reporting2.Patch P 
INNER JOIN Reporting2.DetectedPatchState PP ON P.ID = PP.PATCHID
INNER JOIN Reporting2.PatchDeployment PD ON PD.DetectedPatchStateId = PP.ID
INNER JOIN Reporting2.AssessedMachineState AMS ON AMS.ID = PP.ASSESSEDMACHINESTATEID
WHERE AMS.AssessedOn > (SELECT CASE WHEN COUNT(LOGDATE) <> 0 THEN DATEADD(DD, -1, MAX(LOGDATE)) ELSE ''''2000-01-01 00:00:00.000'''' END FROM xtrEntityProcessLog 
WHERE  PROCEDURENAME LIKE ''''%CurrentDeployStatus%'''') 
GROUP BY AMS.MACHINEID, P.ID, PP.PRODUCTID) B
ON PD.DetectedPatchStateId = B.MAXAMS and MS.MACHINEID = B.MACHINEID AND  P.ID = B.PATCHID AND PT.ID = B.PRODUCTID 



DELETE FROM #CurrentDeploy FROM #CurrentDeploy CDS 
LEFT OUTER  JOIN
(SELECT AMS.MACHINEID,  P.ID AS PATCHID, PP.PRODUCTID AS PRODUCTID,  
MAX(PD.DEPLOYID) AS MAXDEPLOY
FROM Reporting2.Patch P 
INNER JOIN Reporting2.DetectedPatchState PP ON P.ID = PP.PATCHID
INNER JOIN Reporting2.PatchDeployment PD ON PD.DetectedPatchStateId = PP.ID
INNER JOIN Reporting2.AssessedMachineState AMS ON AMS.ID = PP.ASSESSEDMACHINESTATEID 
WHERE AMS.ASSESSEDON > (SELECT CASE WHEN COUNT(LOGDATE) <> 0 THEN DATEADD(DD, -1, MAX(LOGDATE)) ELSE ''''2000-01-01 00:00:00.000'''' END FROM xtrEntityProcessLog 
WHERE  PROCEDURENAME LIKE ''''%CurrentDeployStatus%'''') 
GROUP BY AMS.MACHINEID, P.ID, PP.PRODUCTID) C
ON CDS.DEPLOYID = C.MAXDEPLOY and CDS.MACHINEID = C.MACHINEID AND  CDS.PATCHID = C.PATCHID AND CDS.PRODUCTID = C.PRODUCTID 
WHERE MAXDEPLOY IS NULL



MERGE  xtrCurrentPatchStatus T
USING #CurrentDeploy S
ON T.MACHINEID = S.MACHINEID AND T.PATCHID = S.PATCHID AND T.PRODUCTID = S.PRODUCTID 
WHEN MATCHED THEN 
UPDATE SET DEPLOYID = S.DEPLOYID, DetectedPatchStateID = S.DetectedPatchStateID

OUTPUT $action INTO #MergeSummary;

SET @RECORDSSTAGED = @@rowcount


INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 

SELECT @BATCHID, GETUTCDATE(),  ''''xtr_CurrentDeployStatus'''', ''''Updated '''' + CAST(ISNULL((SELECT COUNT(0) AS RecordCount FROM  #MergeSummary WHERE CHANGE = ''''UPDATE'''' GROUP BY CHANGE),0) AS NVARCHAR(100)) + '''' records''''

DROP TABLE #MergeSummary
DROP TABLE #CurrentDeploy



END TRY

BEGIN CATCH

	EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME


END CATCH''



EXEC sp_executesql @SQLProc






SET @SQLProc = ''CREATE PROCEDURE [dbo].[xtr_CurrentPatchStatus]
(
@BATCHID INT,
@RECORDSSTAGED INT OUTPUT)

AS 
DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_CurrentPatchStatus''''
BEGIN TRY

IF OBJECT_ID(''''tempdb..#MergeSummary'''') IS NULL
	CREATE TABLE #MergeSummary (CHANGE VARCHAR(20))

IF OBJECT_ID(''''tempdb..#TempStatus'''') IS NULL
CREATE TABLE #TempStatus(
MACHINEID INT,
BULLETIN VARCHAR(256),
PATCHID INT,
PRODUCTID INT,
ASSESSEDMACHINESTATEID INT,
SCANDATE DATETIME,
INSTALLSTATEID INT,
OSID INT,
DEPLOYID INT NULL,
DETECTEDPATCHSTATEID INT NULL,
OCCURENCES INT
)

IF OBJECT_ID(''''tempdb..#CurrentPatchStatus'''') IS NULL
CREATE TABLE #CurrentPatchStatus(
MACHINEID INT,
BULLETIN VARCHAR(256),
PATCHID INT,
PRODUCTID INT,
ASSESSEDMACHINESTATEID INT,
SCANDATE DATETIME,
INSTALLSTATEID INT,
OSID INT,
DEPLOYID INT NULL,
DETECTEDPATCHSTATEID INT NULL
)



INSERT INTO #CurrentPatchStatus
SELECT MS.MACHINEID AS MACHINEID, P.BULLETIN AS BULLETIN, DPS.PATCHID AS PATCHID, PT.ID AS PRODUCTID, DPS.ASSESSEDMACHINESTATEID AS ASSESSEDMACHINESTATEID, 
PS.STARTEDON AS SCANDATE, DPS.INSTALLSTATEID AS INSTALLSTATE, SM.SMACHOSID AS OSID, NULL, NULL
FROM Reporting2.AssessedMachineState MS
LEFT OUTER JOIN
Reporting2.PatchScan PS ON PS.ID = MS.PATCHSCANID
LEFT OUTER JOIN
Reporting2.Machine MA ON MS.MACHINEID = MA.ID
LEFT OUTER JOIN
Reporting2.DetectedPatchState DPS ON MS.ID = DPS.ASSESSEDMACHINESTATEID
LEFT OUTER JOIN
DBO.ScanMachines SM ON DPS.ASSESSEDMACHINESTATEID = SM.SMACHID
LEFT OUTER JOIN
Reporting2.Patch P ON DPS.PATCHID = P.ID
LEFT OUTER JOIN
Reporting2.VendorSeverity VS ON VS.ID = P.VENDORSEVERITYID
LEFT OUTER JOIN
Reporting2.Product PT ON DPS.PRODUCTID = PT.ID
INNER JOIN(
 SELECT AMS.MACHINEID,  P.ID AS PATCHID, PP.PRODUCTID AS PRODUCTID,  MAX(PP.ASSESSEDMACHINESTATEID) AS MAXAMS
FROM Reporting2.Patch P 
INNER JOIN Reporting2.DetectedPatchState PP ON P.ID = PP.PATCHID
INNER JOIN Reporting2.AssessedMachineState AMS ON AMS.ID = PP.ASSESSEDMACHINESTATEID
GROUP BY AMS.MACHINEID, P.ID, PP.PRODUCTID) B
ON DPS.ASSESSEDMACHINESTATEID = B.MAXAMS and MS.MACHINEID = B.MACHINEID AND  P.ID = B.PATCHID AND PT.ID = B.PRODUCTID 
WHERE PS.STARTEDON > (SELECT CASE WHEN COUNT(LOGDATE) <> 0 THEN DATEADD(DD, -1, MAX(LOGDATE)) ELSE ''''2010-01-01 00:00:00.000'''' END FROM xtrEntityProcessLog ) 


DELETE #CurrentPatchStatus FROM #CurrentPatchStatus CPS
WHERE CPS.ASSESSEDMACHINESTATEID NOT IN 
(SELECT B.SMACHID FROM 
  (SELECT DISTINCT SM.SMACHMMKEY, SM.SMACHID, A.SMACHOSID
  FROM DBO.ScanMachines SM
  INNER JOIN
  (SELECT SMACHMMKEY, MAX(SMACHOSID) AS   SMACHOSID
  FROM DBO.ScanMachines 
   GROUP BY SMACHMMKEY) A 
  ON A.SMACHMMKEY = SM.SMACHMMKEY 
  AND A.SMACHOSID = SM.SMACHOSID 
   WHERE SM.SMACHOSID IS NOT NULL
) B
)


DELETE xtrCurrentPatchStatus FROM xtrCurrentPatchStatus CPS
WHERE CPS.ASSESSEDMACHINESTATEID NOT IN 
(SELECT B.SMACHID FROM 
  (SELECT DISTINCT SM.SMACHMMKEY, SM.SMACHID, A.SMACHOSID
  FROM DBO.ScanMachines SM
  INNER JOIN
  (SELECT SMACHMMKEY, MAX(SMACHOSID) AS   SMACHOSID
  FROM DBO.ScanMachines 
   GROUP BY SMACHMMKEY) A 
  ON A.SMACHMMKEY = SM.SMACHMMKEY 
  AND A.SMACHOSID = SM.SMACHOSID 
   WHERE SM.SMACHOSID IS NOT NULL
) B
)

INSERT INTO  #TempStatus 
SELECT [MACHINEID], [BULLETIN], [PATCHID], [PRODUCTID], [ASSESSEDMACHINESTATEID], [SCANDATE], [INSTALLSTATEID], [OSID], [DEPLOYID], [DETECTEDPATCHSTATEID], COUNT(*) OCCURENCES 
  FROM xtrCurrentPatchStatus
GROUP BY  [MACHINEID], [BULLETIN], [PATCHID], [PRODUCTID], [ASSESSEDMACHINESTATEID], [SCANDATE], [INSTALLSTATEID], [OSID], [DEPLOYID], [DETECTEDPATCHSTATEID]
HAVING COUNT(*) > 1;

DELETE xtrCurrentPatchStatus FROM xtrCurrentPatchStatus CPS 
INNER JOIN #TempStatus  TS
ON TS.MACHINEID = CPS.MACHINEID AND TS.PATCHID = CPS.PATCHID 
AND TS.PRODUCTID = CPS.PRODUCTID AND TS.ASSESSEDMACHINESTATEID = CPS.ASSESSEDMACHINESTATEID


INSERT INTO xtrCurrentPatchStatus (MACHINEID, BULLETIN, PATCHID, PRODUCTID, ASSESSEDMACHINESTATEID, SCANDATE, INSTALLSTATEID, OSID, DEPLOYID, DETECTEDPATCHSTATEID)
SELECT [MACHINEID],[BULLETIN],[PATCHID],[PRODUCTID],[ASSESSEDMACHINESTATEID], [SCANDATE], [INSTALLSTATEID], [OSID], [DEPLOYID], [DETECTEDPATCHSTATEID]
FROM #TempStatus 

TRUNCATE TABLE #TempStatus 



INSERT INTO  #TempStatus 
SELECT [MACHINEID], [BULLETIN], [PATCHID], [PRODUCTID], [ASSESSEDMACHINESTATEID], [SCANDATE], [INSTALLSTATEID], [OSID], [DEPLOYID], [DETECTEDPATCHSTATEID], COUNT(*) OCCURENCES 
  FROM #CurrentPatchStatus
GROUP BY  [MACHINEID], [BULLETIN], [PATCHID], [PRODUCTID], [ASSESSEDMACHINESTATEID], [SCANDATE], [INSTALLSTATEID], [OSID], [DEPLOYID], [DETECTEDPATCHSTATEID]
HAVING COUNT(*) > 1;

DELETE #CurrentPatchStatus FROM #CurrentPatchStatus CPS 
INNER JOIN #TempStatus  TS
ON TS.MACHINEID = CPS.MACHINEID AND TS.PATCHID = CPS.PATCHID 
AND TS.PRODUCTID = CPS.PRODUCTID AND TS.ASSESSEDMACHINESTATEID = CPS.ASSESSEDMACHINESTATEID


INSERT INTO #CurrentPatchStatus (MACHINEID, BULLETIN, PATCHID, PRODUCTID, ASSESSEDMACHINESTATEID, SCANDATE, INSTALLSTATEID, OSID, DEPLOYID, DETECTEDPATCHSTATEID)
SELECT [MACHINEID],[BULLETIN],[PATCHID],[PRODUCTID],[ASSESSEDMACHINESTATEID], [SCANDATE], [INSTALLSTATEID], [OSID], [DEPLOYID], [DETECTEDPATCHSTATEID]
FROM #TempStatus 



MERGE xtrCurrentPatchStatus T
USING #CurrentPatchStatus S ON T.MACHINEID=S.MACHINEID AND T.PATCHID=S.PATCHID AND T.PRODUCTID=S.PRODUCTID
WHEN NOT MATCHED BY TARGET 
THEN 
INSERT (MACHINEID, BULLETIN, PATCHID, PRODUCTID, ASSESSEDMACHINESTATEID, SCANDATE, INSTALLSTATEID, OSID, DEPLOYID, DETECTEDPATCHSTATEID)
VALUES (S.MACHINEID, S.BULLETIN, S.PATCHID, S.PRODUCTID, S.ASSESSEDMACHINESTATEID, S.SCANDATE, S.INSTALLSTATEID, S.OSID, S.DEPLOYID, S.DETECTEDPATCHSTATEID)
WHEN MATCHED THEN
UPDATE SET T.ASSESSEDMACHINESTATEID = S.ASSESSEDMACHINESTATEID, T.SCANDATE = S.SCANDATE, T.INSTALLSTATEID = S.INSTALLSTATEID



OUTPUT $action INTO #MergeSummary;

SET @RECORDSSTAGED = @@rowcount


INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 

SELECT @BATCHID, GETUTCDATE(),  ''''xtr_CurrentPatchStatus'''', ''''Inserted '''' + CAST(ISNULL((SELECT COUNT(0) AS RecordCount FROM  #MergeSummary WHERE CHANGE = ''''INSERT'''' GROUP BY CHANGE),0) AS NVARCHAR(100)) + '''' records''''
UNION
SELECT @BATCHID, GETUTCDATE(),  ''''xtr_CurrentPatchStatus'''', ''''Updated '''' + CAST(ISNULL((SELECT COUNT(0) AS RecordCount FROM  #MergeSummary WHERE CHANGE = ''''UPDATE'''' GROUP BY CHANGE),0) AS NVARCHAR(100)) + '''' records''''

DROP TABLE #MergeSummary
DROP TABLE #CurrentPatchStatus
DROP TABLE #TempStatus


END TRY

BEGIN CATCH

	EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME


END CATCH''

EXEC sp_executesql @SQLProc

SET @SQLProc = ''
CREATE PROCEDURE [dbo].[xtr_DeleteCPatches] (
	@PATCHCODE NVARCHAR(256), @BATCHID INT

)
AS
DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_DeleteCPatches''''
DECLARE @SQL NVARCHAR(max)

BEGIN TRY

IF (@PATCHCODE <> ''''MSNS%'''')
BEGIN
SET @SQL = ''''DELETE  xtrCurrentPatchStatus FROM xtrCurrentPatchStatus C
			INNER JOIN
			(SELECT A.ID,  A.MACHINEID, PATCHID, PRODUCTID,   MAX(ASSESSEDMACHINESTATEID) AS MAXAMS , 
			SCANDATE, INSTALLSTATEID
			FROM xtrCurrentPatchStatus A
			INNER JOIN 
			(SELECT DISTINCT MACHINEID, MAX(ASSESSEDMACHINESTATEID) AS MAXID
			FROM xtrCurrentPatchStatus
			WHERE BULLETIN LIKE '''''''''''' + @PATCHCODE + '''''''''''' AND INSTALLSTATEID = 3
			GROUP BY MACHINEID ) B
			ON A.MACHINEID = B.MACHINEID AND A.ASSESSEDMACHINESTATEID < B.MAXID
			WHERE BULLETIN LIKE '''''''''''' + @PATCHCODE + '''''''''''' AND INSTALLSTATEID = 4
			GROUP BY  A.MACHINEID,PATCHID, PRODUCTID,   SCANDATE,  ID, INSTALLSTATEID) D
			ON C.PATCHID = d.PATCHID AND C.PRODUCTID = D.PRODUCTID AND C.ASSESSEDMACHINESTATEID = D.MAXAMS''''


			EXEC sp_executesql @SQL

			RETURN CAST(@@rowcount AS INT)
END
ELSE
BEGIN
SET @SQL = ''''DELETE  xtrCurrentPatchStatus FROM xtrCurrentPatchStatus C
			INNER JOIN
			(SELECT A.ID,  A.MACHINEID, PATCHID, PRODUCTID,   MAX(ASSESSEDMACHINESTATEID) AS MAXAMS , 
			SCANDATE, INSTALLSTATEID
			FROM xtrCurrentPatchStatus A
			INNER JOIN 
			(SELECT DISTINCT MACHINEID, MAX(ASSESSEDMACHINESTATEID) AS MAXID
			FROM xtrCurrentPatchStatus
			WHERE BULLETIN LIKE '''''''''''' + @PATCHCODE + '''''''''''' AND LEN(BULLETIN) < 19 AND INSTALLSTATEID = 3
			GROUP BY MACHINEID ) B
			ON A.MACHINEID = B.MACHINEID AND A.ASSESSEDMACHINESTATEID < B.MAXID
			WHERE BULLETIN LIKE '''''''''''' + @PATCHCODE + '''''''''''' AND LEN(BULLETIN) < 19 AND INSTALLSTATEID = 4
			GROUP BY  A.MACHINEID,PATCHID, PRODUCTID,   SCANDATE,  ID, INSTALLSTATEID) D
			ON C.PATCHID = d.PATCHID AND C.PRODUCTID = D.PRODUCTID AND C.ASSESSEDMACHINESTATEID = D.MAXAMS''''


			EXEC sp_executesql @SQL

			RETURN CAST(@@rowcount AS INT)
END
			
END TRY

BEGIN CATCH
	EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
	
END CATCH''

EXEC sp_executesql @SQLProc

SET @SQLProc = ''
CREATE PROCEDURE [dbo].[xtr_DeleteObsPatches] (
	@PATCHCODE NVARCHAR(256), @BATCHID INT

)
AS
DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_DeleteObsPatches''''
DECLARE @SQL NVARCHAR(max)

BEGIN TRY


SET @SQL = ''''DELETE xtrCurrentPatchStatus FROM xtrCurrentPatchStatus C
			INNER JOIN
			(SELECT MACHINEID, PATCHID, PRODUCTID,   ASSESSEDMACHINESTATEID, 
			SCANDATE, INSTALLSTATEID
			FROM xtrCurrentPatchStatus 
			WHERE (BULLETIN LIKE '''''''''''' + @PATCHCODE + '''''''''''')
			AND INSTALLSTATEID = 4) DP
			ON C.MACHINEID = DP.MACHINEID
			AND C.PATCHID = DP.PATCHID
			AND C.PRODUCTID = DP.PRODUCTID
			AND C.ASSESSEDMACHINESTATEID = DP.ASSESSEDMACHINESTATEID''''

			EXEC sp_executesql @SQL

			RETURN CAST(@@rowcount AS INT)

END TRY

BEGIN CATCH
	EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
	
END CATCH''

EXEC sp_executesql @SQLProc

SET @SQLProc = ''
CREATE PROCEDURE [dbo].[xtr_DeleteODPatches] (
	@BATCHID INT

)
AS
DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_DeleteODPatches''''
DECLARE @SQL NVARCHAR(max)

BEGIN TRY


SET @SQL = ''''DELETE xtrCurrentPatchStatus FROM xtrCurrentPatchStatus C
			INNER JOIN 
			(SELECT MACHINEID, MAX(SCANDATE) AS SCANDATE FROM xtrCurrentPatchStatus
			GROUP BY MACHINEID) M
			ON M.MACHINEID = C.MACHINEID
			AND C.SCANDATE < DATEADD(dd,-90, M.SCANDATE)
			WHERE C.INSTALLSTATEID IN (4,6)''''

			EXEC sp_executesql @SQL

			RETURN CAST(@@rowcount AS INT)

END TRY

BEGIN CATCH
	EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
	
END CATCH''

EXEC sp_executesql @SQLProc






SET @SQLProc = ''
CREATE PROCEDURE [dbo].[xtr_DistinctAssessed](
@BATCHID INT)


AS

DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_DistinctAssessed''''

BEGIN TRY
TRUNCATE TABLE xtrDistinctAssessed

INSERT INTO xtrDistinctAssessed

SELECT machineid, MONTH([AssessedOn]) M, year([AssessedOn]) y, max([AssessedOn]) Assessedon
  FROM [Reporting2].[AssessedMachineState]
  WHERE AssessedOn IS NOT NULL 
  group by machineid, MONTH([AssessedOn]) , year([AssessedOn]) 

RETURN CAST(@@rowcount AS INT)

END TRY

BEGIN CATCH
	EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
	
END CATCH''

EXEC sp_executesql @SQLProc



SET @SQLProc = ''
CREATE PROCEDURE [dbo].[xtr_DistinctDeployed](
@BATCHID INT)


AS

DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_DistinctDeployed''''

BEGIN TRY
TRUNCATE TABLE xtrDistinctDeployed

INSERT INTO xtrDistinctDeployed

SELECT AMS.machineid, MONTH(PD.DeployStartedOn) M, year(PD.DeployStartedOn) y, max(PD.DeployStartedOn) DeployedOn
  FROM [Reporting2].[AssessedMachineState] AMS join [Reporting2].[DetectedPatchState] DPS on AMS.Id = DPS.AssessedMachineStateId
  join [Reporting2].[PatchDeployment] PD on DPS.Id = PD.DetectedPatchStateId
  WHERE PD.DeployStartedOn IS NOT NULL 
  group by AMS.machineid, MONTH(PD.DeployStartedOn) , year(PD.DeployStartedOn)

RETURN CAST(@@rowcount AS INT)

END TRY

BEGIN CATCH
	EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
	
END CATCH''

EXEC sp_executesql @SQLProc





SET @SQLProc = ''
CREATE PROCEDURE [dbo].[xtr_DistinctPatched](
@BATCHID INT)


AS

DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_DistinctPatched''''

BEGIN TRY
TRUNCATE TABLE xtrDistinctPatched

INSERT INTO xtrDistinctPatched

SELECT AMS.machineid, MONTH(DPS.InstalledOn) M, year(DPS.InstalledOn) y, max(DPS.InstalledOn) PatchedOn
  FROM [Reporting2].[AssessedMachineState] AMS join [Reporting2].[DetectedPatchState] DPS on AMS.Id = DPS.AssessedMachineStateId
  WHERE DPS.InstalledOn IS NOT NULL 
  group by AMS.machineid, MONTH(DPS.InstalledOn) , year(DPS.InstalledOn)

RETURN CAST(@@rowcount AS INT)

END TRY

BEGIN CATCH
	EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
	
END CATCH''

EXEC sp_executesql @SQLProc







SET @SQLProc = ''

CREATE PROCEDURE [dbo].[xtr_LinuxCurrentPatchCount](
@BATCHID INT)

AS

DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_LinuxCurrentPatchCount''''

BEGIN TRY
TRUNCATE TABLE xtrLinuxCurrentPatchCount

INSERT INTO xtrLinuxCurrentPatchCount

SELECT AMS.MACHINEID AS MACHINE, VS.VALUE AS SEVERITY,
SUM(CASE WHEN INSTALLSTATEID = 3 THEN 1 ELSE 0 END) AS INSTALLED,
SUM(CASE WHEN INSTALLSTATEID = 4 THEN 1 ELSE 0 END) AS NOTINSTALLED,
SUM(CASE WHEN INSTALLSTATEID = 6 THEN 1 ELSE 0 END) AS MISSINGSERVICEPACK,
SUM(CASE WHEN INSTALLSTATEID = 2 THEN 1 ELSE 0 END) AS INFORMATIONAL

FROM xtrLinuxCurrentPatchStatus LCPS
LEFT OUTER JOIN 
Reporting2.AssessedMachineState AMS ON AMS.ID = LCPS.ASSESSEDMACHINESTATEID
LEFT OUTER JOIN
Reporting2.LinuxPatch LP ON LCPS.PATCHID = LP.ID
LEFT OUTER JOIN
Reporting2.VendorSeverity VS ON VS.ID = LP.VENDORSEVERITYID
GROUP BY AMS.MACHINEID,  VS.VALUE


RETURN CAST(@@rowcount AS INT)


END TRY

BEGIN CATCH
	EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
	
END CATCH''

EXEC sp_executesql @SQLProc

SET @SQLProc = ''CREATE PROCEDURE [dbo].[xtr_LinuxCurrentPatchStatus]
(
@BATCHID INT,
@RECORDSSTAGED INT OUTPUT)
	--EXEC sp_SyncData
AS 
DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_LinuxCurrentPatchStatus''''
BEGIN TRY

IF OBJECT_ID(''''tempdb..#MergeSummary'''') IS NULL
	CREATE TABLE #MergeSummary (CHANGE VARCHAR(20))


DECLARE @LinuxTempStatus TABLE (
MACHINEID INT,
PATCH VARCHAR(256),
PATCHID INT,
ASSESSEDMACHINESTATEID INT,
SCANDATE DATETIME,
INSTALLSTATEID INT,
OSID INT,
OCCURENCES INT
)

DECLARE @LinuxCurrentPatchStatus TABLE (
MACHINEID INT,
PATCH VARCHAR(256),
PATCHID INT,
ASSESSEDMACHINESTATEID INT,
SCANDATE DATETIME,
INSTALLSTATEID INT,
OSID INT

)



INSERT INTO @LinuxCurrentPatchStatus
SELECT   MS.MACHINEID AS MACHINEID, LP.Name AS PATCH ,  LDPS.LINUXPATCHID AS PATCHID, LDPS.ASSESSEDMACHINESTATEID AS ASSESSEDMACHINESTATEID, 
PS.STARTEDON AS SCANDATE , LDPS.LinuxINSTALLSTATEID AS INSTALLSTATE, SM.PLATFORMID AS OSID
FROM Reporting2.AssessedMachineState MS
LEFT OUTER JOIN
Reporting2.PatchScan PS ON PS.ID = MS.PATCHSCANID
LEFT OUTER JOIN
Reporting2.LinuxDetectedPatchState LDPS ON MS.ID = LDPS.ASSESSEDMACHINESTATEID
LEFT OUTER JOIN
Reporting2.LINUXPATCH LP ON LDPS.LINUXPATCHID = LP.ID
LEFT OUTER JOIN
Reporting2.VendorSeverity VS ON VS.ID = LP.VENDORSEVERITYID
LEFT OUTER JOIN
DBO.ScanMachines SM ON LDPS.ASSESSEDMACHINESTATEID = SM.SMACHID
INNER JOIN(
 SELECT AMS.MACHINEID,  LP.ID AS PATCHID,  MAX(LDPS.ASSESSEDMACHINESTATEID) AS MAXAMS
FROM Reporting2.LINUXPATCH LP 
INNER JOIN Reporting2.LinuxDetectedPatchState LDPS ON LP.ID = LDPS.LINUXPATCHID
INNER JOIN Reporting2.AssessedMachineState AMS ON AMS.ID = LDPS.ASSESSEDMACHINESTATEID
GROUP BY AMS.MACHINEID, LP.ID ) B
ON LDPS.ASSESSEDMACHINESTATEID = B.MAXAMS and MS.MACHINEID = B.MACHINEID AND  LP.ID = B.PATCHID 
WHERE PS.STARTEDON > (SELECT CASE WHEN COUNT(LOGDATE) <> 0 THEN DATEADD(DD, -1, MAX(LOGDATE)) ELSE ''''2000-01-01 00:00:00.000'''' END FROM xtrEntityProcessLog 
WHERE  PROCEDURENAME LIKE ''''%Linux%'''') 


INSERT INTO  @LinuxTempStatus 
SELECT [MACHINEID],[PATCH] ,[PATCHID], [ASSESSEDMACHINESTATEID], [SCANDATE], [INSTALLSTATEID], [OSID], COUNT(*) OCCURENCES 
  FROM @LinuxCurrentPatchStatus
GROUP BY  [MACHINEID] ,[PATCH]  ,[PATCHID]  , [ASSESSEDMACHINESTATEID], [SCANDATE], [INSTALLSTATEID], [OSID]
HAVING COUNT(*) > 1;

DELETE @LinuxCurrentPatchStatus FROM @LinuxCurrentPatchStatus CPS 
INNER JOIN @LinuxTempStatus  TS
ON TS.MACHINEID = CPS.MACHINEID AND TS.PATCHID = CPS.PATCHID 
AND TS.ASSESSEDMACHINESTATEID = CPS.ASSESSEDMACHINESTATEID


INSERT INTO @LinuxCurrentPatchStatus (MACHINEID, PATCH, PATCHID,  ASSESSEDMACHINESTATEID, SCANDATE, INSTALLSTATEID, OSID)
SELECT [MACHINEID],[PATCH],[PATCHID],[ASSESSEDMACHINESTATEID], [SCANDATE], [INSTALLSTATEID], [OSID] 
FROM @LinuxTempStatus 



MERGE xtrLINUXCURRENTPATCHSTATUS T
USING @LinuxCurrentPatchStatus S ON T.MACHINEID=S.MACHINEID AND T.PATCHID=S.PATCHID 
WHEN NOT MATCHED BY TARGET 
THEN 
INSERT (MACHINEID, PATCH, PATCHID, ASSESSEDMACHINESTATEID, SCANDATE, INSTALLSTATEID, OSID)
VALUES (S.MACHINEID, S.PATCH, S.PATCHID, S.ASSESSEDMACHINESTATEID, S.SCANDATE, S.INSTALLSTATEID, S.OSID)
WHEN MATCHED THEN
UPDATE SET T.ASSESSEDMACHINESTATEID = S.ASSESSEDMACHINESTATEID, T.SCANDATE = S.SCANDATE, T.INSTALLSTATEID = S.INSTALLSTATEID, T.OSID = S.OSID
OUTPUT $action INTO #MergeSummary;



SET @RECORDSSTAGED = @@rowcount


INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 

SELECT @BATCHID, GETUTCDATE(),  ''''xtr_LinuxCurrentPatchStatus'''', ''''Inserted '''' + CAST(ISNULL((SELECT COUNT(0) AS RecordCount FROM  #MergeSummary WHERE CHANGE = ''''INSERT'''' GROUP BY CHANGE),0) AS NVARCHAR(100)) + '''' records''''
UNION
SELECT @BATCHID, GETUTCDATE(),  ''''xtr_LinuxCurrentPatchStatus'''', ''''Updated '''' + CAST(ISNULL((SELECT COUNT(0) AS RecordCount FROM  #MergeSummary WHERE CHANGE = ''''UPDATE'''' GROUP BY CHANGE),0) AS NVARCHAR(100)) + '''' records''''

DROP TABLE #MergeSummary



END TRY

BEGIN CATCH

	EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME


END CATCH''

EXEC sp_executesql @SQLProc

SET @SQLProc = ''CREATE PROCEDURE [dbo].[xtr_LinuxDeleteCPatches] (
	@PATCHCODE NVARCHAR(256), @BATCHID INT

)
AS
DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_LinuxDeleteCPatches''''
DECLARE @SQL NVARCHAR(max)

BEGIN TRY

SET @SQL = ''''DELETE  xtrLinuxCurrentPatchStatus FROM xtrLinuxCurrentPatchStatus C
			INNER JOIN
			(SELECT A.ID,  A.MACHINEID, PATCHID,   MAX(ASSESSEDMACHINESTATEID) AS MAXAMS , 
			SCANDATE, INSTALLSTATEID
			FROM xtrLinuxCurrentPatchStatus A
			INNER JOIN 
			(SELECT DISTINCT MACHINEID, MAX(ASSESSEDMACHINESTATEID) AS MAXID
			FROM xtrLinuxCurrentPatchStatus
			WHERE PATCH LIKE '''''''''''' + @PATCHCODE + '''''''''''' AND INSTALLSTATEID = 3
			GROUP BY MACHINEID ) B
			ON A.MACHINEID = B.MACHINEID AND A.ASSESSEDMACHINESTATEID < B.MAXID
			WHERE PATCH LIKE '''''''''''' + @PATCHCODE + '''''''''''' AND INSTALLSTATEID = 4
			GROUP BY  A.MACHINEID,PATCHID,   SCANDATE,  ID, INSTALLSTATEID) D
			ON C.PATCHID = d.PATCHID  AND C.ASSESSEDMACHINESTATEID = D.MAXAMS''''


			EXEC sp_executesql @SQL

			RETURN CAST(@@rowcount AS INT)

	

			
END TRY

BEGIN CATCH
	EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
	
END CATCH''

EXEC sp_executesql @SQLProc



SET @SQLProc = ''
CREATE PROCEDURE [dbo].[xtr_LinuxDistinctDeployed](
@BATCHID INT)


AS

DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_LinuxDistinctDeployed''''

BEGIN TRY
TRUNCATE TABLE xtrLinuxDistinctDeployed

INSERT INTO xtrLinuxDistinctDeployed

SELECT AMS.machineid, MONTH(PD.InstallStartedOn) M, year(PD.InstallStartedOn) y, max(PD.InstallStartedOn) DeployedOn
  FROM [Reporting2].[AssessedMachineState] AMS join [Reporting2].[LinuxDetectedPatchState] DPS on AMS.Id = DPS.AssessedMachineStateId
  join [Reporting2].[LINUXPatchDeployment] PD on DPS.Id = PD.LinuxDetectedPatchStateId
  WHERE PD.InstallStartedOn IS NOT NULL 
  group by AMS.machineid, MONTH(PD.InstallStartedOn) , year(PD.InstallStartedOn)

RETURN CAST(@@rowcount AS INT)

END TRY

BEGIN CATCH
	EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
	
END CATCH''

EXEC sp_executesql @SQLProc







SET @SQLProc = ''CREATE PROCEDURE [dbo].[xtr_UpdateData]

AS

BEGIN TRY

DECLARE @CumulativePatches TABLE (
ID INT,
PATCHCODE VARCHAR(256),
ACTIVE INT
)
DECLARE @ENTITYNAME VARCHAR(100) = ''''xtr_UpdateData''''
DECLARE @STAGEDRECORDCOUNT INT = 0
DECLARE @ID INT, @PATCHCODE VARCHAR(256)
DECLARE @BATCHID INT = (SELECT ISNULL(max(BATCHID),0) + 1 FROM dbo.xtrEntityProcessLog)
DECLARE @DELETEDRECORDS INT = 0
DECLARE @DELETEDOBSRECORDS INT = 0
DECLARE @DELETEDODRECORDS INT = 0
DECLARE @RETURNVALUE INT = 0

-- Calls the procedure xtr_CurrentPatchStatus which gets a list of all patches,
-- the install state of each patch the last time it was detected and the datetime of that detection

BEGIN
EXEC xtr_CurrentPatchStatus @BATCHID, @RECORDSSTAGED = @STAGEDRECORDCOUNT OUTPUT
END

BEGIN
EXEC xtr_CurrentDeployStatus @BATCHID, @RECORDSSTAGED = @STAGEDRECORDCOUNT OUTPUT
END

-- Calls the procedure xtr_DeleteCPatches and removes any patch that is in the cumulative list that has a
-- later version of that patch installed

INSERT INTO @CumulativePatches
SELECT 
	ID,
	PATCHCODE,
	ACTIVE
FROM dbo.xtrCumulativePatches
WHERE ACTIVE = 1

WHILE (SELECT COUNT(0) FROM @CumulativePatches) > 0
BEGIN
	BEGIN TRY
	
		SELECT TOP 1 
			@ID = ID,
			@PATCHCODE = PATCHCODE
 		FROM @CumulativePatches
		
		EXEC @RETURNVALUE = xtr_DeleteCPatches @PATCHCODE, @BATCHID
		SET @DELETEDRECORDS = @DELETEDRECORDS + @RETURNVALUE
		DELETE FROM @CumulativePatches WHERE PATCHCODE = @PATCHCODE

	END TRY

		BEGIN CATCH
		DELETE FROM @CumulativePatches WHERE PATCHCODE = @PATCHCODE
		END CATCH
END
			INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 
			VALUES (@BATCHID, GETUTCDATE(), ''''xtr_DeleteCPatches'''', ''''Deleted '''' + CAST(@DELETEDRECORDS AS NVARCHAR(100)) + '''' records'''')
END TRY
	
BEGIN CATCH
EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
END CATCH

-- Calls the procedure xtr_DeleteObsPatches and removes any obsolete patch that is in the cumulative list

BEGIN TRY
INSERT INTO @CumulativePatches
SELECT 
	ID,
	PATCHCODE,
	ACTIVE
FROM dbo.xtrCumulativePatches
WHERE ACTIVE = 2

WHILE (SELECT COUNT(0) FROM @CumulativePatches) > 0
BEGIN
	BEGIN TRY
	
		SELECT TOP 1 
			@ID = ID,
			@PATCHCODE = PATCHCODE
 		FROM @CumulativePatches
		
		EXEC @RETURNVALUE = xtr_DeleteObsPatches @PATCHCODE, @BATCHID
		SET @DELETEDOBSRECORDS = @DELETEDOBSRECORDS + @RETURNVALUE
		DELETE FROM @CumulativePatches WHERE PATCHCODE = @PATCHCODE

	END TRY

		BEGIN CATCH
		DELETE FROM @CumulativePatches WHERE PATCHCODE = @PATCHCODE
		END CATCH
END
			INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 
			VALUES (@BATCHID, GETUTCDATE(), ''''xtr_DeleteObsPatches'''', ''''Deleted '''' + CAST(@DELETEDOBSRECORDS AS NVARCHAR(100)) + '''' records'''')
END TRY
	
BEGIN CATCH
EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
END CATCH
-- Calls the procedure xtr_DeleteODPatches and removes any patch that has not been detected in the last 90 days where the machine
-- has been scanned in the last 90 days
BEGIN
		
		EXEC @RETURNVALUE = xtr_DeleteODPatches @BATCHID
		SET @DELETEDODRECORDS =  @RETURNVALUE

			INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 
			VALUES (@BATCHID, GETUTCDATE(), ''''xtr_DeleteODPatches'''', ''''Deleted '''' + CAST(@DELETEDODRECORDS AS NVARCHAR(100)) + '''' records'''')
END

BEGIN TRY
EXEC @RETURNVALUE = xtr_CurrentPatchCount @BATCHID

INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 
VALUES (@BATCHID, GETUTCDATE(), ''''xtr_CurrentPatchCount'''', ''''Inserted '''' + CAST(@RETURNVALUE AS NVARCHAR(100)) + '''' records'''')

END TRY

BEGIN CATCH
EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
END CATCH



BEGIN TRY
EXEC @RETURNVALUE = xtr_DistinctAssessed @BATCHID

INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 
VALUES (@BATCHID, GETUTCDATE(), ''''xtr_DistinctAssessed'''', ''''Inserted '''' + CAST(@RETURNVALUE AS NVARCHAR(100)) + '''' records'''')

END TRY

BEGIN CATCH
EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
END CATCH


BEGIN TRY
EXEC @RETURNVALUE = xtr_DistinctDeployed @BATCHID

INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 
VALUES (@BATCHID, GETUTCDATE(), ''''xtr_DistinctDeployed'''', ''''Inserted '''' + CAST(@RETURNVALUE AS NVARCHAR(100)) + '''' records'''')

END TRY

BEGIN CATCH
EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
END CATCH

BEGIN TRY
EXEC @RETURNVALUE = xtr_DistinctPatched @BATCHID

INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 
VALUES (@BATCHID, GETUTCDATE(), ''''xtr_DistinctPatched'''', ''''Inserted '''' + CAST(@RETURNVALUE AS NVARCHAR(100)) + '''' records'''')

END TRY

BEGIN CATCH
EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
END CATCH








BEGIN TRY

DECLARE @LinuxCumulativePatches TABLE (
ID INT,
PATCHCODE VARCHAR(256),
ACTIVE INT
)
SET @ENTITYNAME = ''''xtr_LinuxUpdateData''''
SET  @STAGEDRECORDCOUNT = 0
SET @ID = 0
SET @PATCHCODE = ''''''''
SET @BATCHID = (SELECT ISNULL(max(BATCHID),0) + 1 FROM dbo.xtrEntityProcessLog)
SET @DELETEDRECORDS = 0
SET @RETURNVALUE = 0
BEGIN

EXEC xtr_LinuxCurrentPatchStatus @BATCHID, @RECORDSSTAGED = @STAGEDRECORDCOUNT OUTPUT

END


INSERT INTO @LinuxCumulativePatches
SELECT 
	ID,
	PATCHCODE,
	ACTIVE
FROM dbo.xtrLinuxCumulativePatches
WHERE ACTIVE = 1


WHILE (SELECT COUNT(0) FROM @LinuxCumulativePatches) > 0
BEGIN
	BEGIN TRY
	
		SELECT TOP 1 
			@ID = ID,
			@PATCHCODE = PATCHCODE
 		FROM @LinuxCumulativePatches


		
		EXEC @RETURNVALUE = xtr_LinuxDeleteCPatches @PATCHCODE, @BATCHID
		SET @DELETEDRECORDS = @DELETEDRECORDS + @RETURNVALUE
		DELETE FROM @LinuxCumulativePatches WHERE PATCHCODE = @PATCHCODE


	END TRY

		BEGIN CATCH
		DELETE FROM @LinuxCumulativePatches WHERE PATCHCODE = @PATCHCODE
		END CATCH


END

			INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 
			VALUES (@BATCHID, GETUTCDATE(), ''''xtr_LinuxDeleteCPatches'''', ''''Deleted '''' + CAST(@DELETEDRECORDS AS NVARCHAR(100)) + '''' records'''')

END TRY
	
BEGIN CATCH
EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
END CATCH



BEGIN TRY
EXEC @RETURNVALUE = xtr_LinuxCurrentPatchCount @BATCHID

INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 
VALUES (@BATCHID, GETUTCDATE(), ''''xtr_LinuxCurrentPatchCount'''', ''''Inserted '''' + CAST(@RETURNVALUE AS NVARCHAR(100)) + '''' records'''')

END TRY

BEGIN CATCH
EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
END CATCH

BEGIN TRY
EXEC @RETURNVALUE = xtr_LinuxDistinctDeployed @BATCHID

INSERT INTO dbo.xtrEntityProcessLog (BATCHID, LOGDATE, PROCEDURENAME, DESCRIPTION) 
VALUES (@BATCHID, GETUTCDATE(), ''''xtr_LinuxDistinctDeployed'''', ''''Inserted '''' + CAST(@RETURNVALUE AS NVARCHAR(100)) + '''' records'''')

END TRY

BEGIN CATCH
EXEC xtr_LogErrorInfo @BATCHID, @ENTITYNAME
END CATCH''

EXEC sp_executesql @SQLProc


END



if @RETRYCOUNT >= @RETRYMAX AND @LASTERROR LIKE ''7303:%''
    raiserror(''The stored procedure xtr_RefreshData was not able to connect to the Security Controls database after multiple retries'',16,1)', 
		@database_name=@DatabaseName, 
		@flags=0















EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'Refresh Data', 
		@step_id=2, 
		@cmdexec_success_code=0, 
		@on_success_action=1, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'DECLARE @RETRYCOUNT INT = 1
DECLARE @RETRYMAX INT = 3
DECLARE @LASTBATCH INT
DECLARE @LASTERROR NVARCHAR(4000)







EXEC dbo.xtr_UpdateData

ALTER INDEX ALL ON dbo.xtrCurrentPatchStatus
REORGANIZE ;
ALTER INDEX ALL ON dbo.xtrLinuxCurrentPatchStatus
REORGANIZE ;

SELECT @LASTBATCH = MAX(BatchId) FROM dbo.xtrEntityProcessLog

SELECT @LASTERROR = ErrorMessage FROM dbo.xtrEntityProcessErrorLog WHERE BatchId = @LASTBATCH

--SELECT @LASTERROR

WHILE (@RETRYCOUNT <= @RETRYMAX AND @LASTERROR LIKE ''7303:%'')
BEGIN
	SET @LASTERROR = NULL

	EXEC dbo.xtr_UpdateData
	
	SELECT @LASTBATCH = MAX(BatchId) FROM dbo.xtrEntityProcessLog
	SELECT @LASTERROR = ErrorMessage FROM dbo.xtrEntityProcessErrorLog WHERE BatchId = @LASTBATCH
	
	SET @RETRYCOUNT = @RETRYCOUNT + 1
END

if @RETRYCOUNT >= @RETRYMAX AND @LASTERROR LIKE ''7303:%''
    raiserror(''The stored procedure xtr_UpdateData was not able to connect to the Security Controls database after multiple retries'',16,1)', 
		@database_name=@DatabaseName, 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_update_job @job_id = @jobId, @start_step_id = 1
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobschedule @job_id=@jobId, @name=N'Every 60 minutes', 
		@enabled=1, 
		@freq_type=4, 
		@freq_interval=1, 
		@freq_subday_type=4, 
		@freq_subday_interval=60, 
		@freq_relative_interval=0, 
		@freq_recurrence_factor=0, 
		@active_start_date=20131204, 
		@active_end_date=99991231, 
		@active_start_time=0, 
		@active_end_time=235959, 
		@schedule_uid=N'b6a028bf-0d35-4c7e-a348-6e58053124d4'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @jobId, @server_name = N'(local)'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
    IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:

GO


